﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace ZEGO
{
    // MARK: - enum

    // Connection state.
    //
    // Description: The state machine that identifies the current connection state.
    //
    // Use cases: It can be used to determine whether the login/logout is successful,
    // and to handle abnormal situations such as network disconnection.
    //
    // Caution: Please use it with the connection event parameter.
    //
    public enum ZIMConnectionState
    {

        // Unconnected state, enter this state before logging in and after logging out. If there is a steady state abnormality in the process of logging in, such as AppID or Token are incorrect, or if the same user name is logged in elsewhere and the local end is kicked out, it will enter this state.
        Disconnected = 0,

        // The state that the connection is being requested. It will enter this state after successful execution login function. The display of the UI is usually performed using this state. If the connection is interrupted due to poor network quality, the SDK will perform an internal retry and will return to this state.
        Connecting = 1,

        // The state that is successfully connected. Entering this state indicates that login successfully and the user can use the SDK functions normally.
        Connected = 2,

        // The state that the reconnecting is being requested. It will enter this state after successful execution login function. The display of the UI is usually performed using this state. If the connection is interrupted due to poor network quality, the SDK will perform an internal retry and will return to this state.
        Reconnecting = 3
    };

    // The event that caused the connection status to change.
    //
    // Description: The reason for the change of the connection state.
    //
    // Use cases: It can be used to determine whether the login/logout is successful,
    // and to handle abnormal situations such as network disconnection.
    //
    // Caution: Please use it with the connection state parameter.
    //
    public enum ZIMConnectionEvent
    {

        // Success.
        Success = 0,

        // The user actively logs in.
        ActiveLogin = 1,

        // Connection timed out.
        LoginTimeout = 2,

        // The network connection is temporarily interrupted.
        LoginInterrupted = 3,

        // Being kicked out.
        KickedOut = 4,

        // Being token expierd.
        TokenExpired = 5,

    };

    public enum ZIMRoomState
    {

        // Unconnected state, enter this state before logging in and after logging out. If there is a steady state abnormality in the process of logging in, such as AppID or Token are incorrect, or if the same user name is logged in elsewhere and the local end is kicked out, it will enter this state.
        Disconnected = 0,

        // The state that the connection is being requested. It will enter this state after successful execution login function. The display of the UI is usually performed using this state. If the connection is interrupted due to poor network quality, the SDK will perform an internal retry and will return to this state.
        Connecting = 1,

        // The state that is successfully connected. Entering this state indicates that login successfully and the user can use the SDK functions normally.
        Connected = 2

    };

    public enum ZIMRoomEvent
    {

        // Success.
        Success = 0,
        NetworkInterrupted = 1,
        NetworkDisconnected = 2,
        RoomNotExist = 3,
        ActiveCreate = 4,
        CreateFailed = 5,
        ActiveEnter = 6,
        EnterFailed = 7,
        KickedOut = 8,
        ConnectTimeout = 9

    };

    public enum ZIMRoomAttributesUpdateAction
    {
        Set = 0,
        Delete = 1
    };

    public enum ZIMGroupAttributesUpdateAction
    {
        Set = 0,
        Delete = 1
    };

    // The priority of the message.
    //
    // Description: Identifies the priority of a message.
    //
    // Use cases: It can be used to set the priority when a message is sent.
    // The higher the priority, the higher the reliability.
    // Low priority may be discarded due to weak network.
    //
    // Caution: The higher the priority, the higher the cost.
    //
    public enum ZIMMessagePriority
    {

        // Description: Low priority.
        //
        // Use cases: Generally used to send unimportant messages such as barrage message in a room.

        Low = 1,

        // Description: Medium priority.
        //
        // Use cases: Generally used to send regular chat messages.

        Medium = 2,

        // Description: High priority.
        //
        // Use cases: Generally used to send important information such as gifts and rewards in a room.

        High = 3

    };

    // The type of the message.
    //
    // Description: Identifies the type of current message.
    //
    // Use cases: It can be used to determine what type of message this message is.
    //
    public enum ZIMMessageType
    {

        Unknown = 0,

        // Normal text message.
        Text = 1,

        // Custom binary message.
        Command = 2,

        Image = 11,

        File = 12,

        Audio = 13,

        Video = 14,

        Barrage = 20,

        System = 30,

        Revoke = 31,

        Custom = 200
    };

    public enum ZIMMediaFileType
    {
        OriginalFile = 1,

        LargeImage = 2,

        Thumbnail = 3,

        VideoFirstFrame = 4
    };

    public enum ZIMMessageDirection { Send = 0, Receive = 1 };

    public enum ZIMMessageSentStatus
    {
        Sending = 0,
        SendSuccess = 1,
        SendFailed = 2
    };

    public enum ZIMMessageReceiptStatus
    {
        Unknown = -1,
        None = 0,
        Processing = 1,
        Done = 2,
        Expired = 3,
        Failed = 4
    };

    public enum ZIMRevokeType
    {
        Unknown = -1,
        TwoWay = 0,
        OneWay = 1
    };

    public enum ZIMMessageRevokeStatus
    {
        Unknown = -1,
        SelfRevoke = 0,
        SystemRevoke = 1,
        ServerApiRevoke = 2,
        GroupAdminRevoke = 3,
        GroupOwnerRevoke = 4
    };

    public enum ZIMMessageOrder
    {
        Descending = 0,
        Ascending = 1
    }

    public enum ZIMConversationType
    {
        Unknown = -1,
        Peer = 0,
        Room = 1,
        Group = 2
    };

    public enum ZIMConversationNotificationStatus
    {
        Notify = 1,
        DoNotDisturb = 2
    };

    public enum ZIMConversationEvent
    {
        Added = 0,
        Updated = 1,
        Disabled = 2
    };

    // 群变更事件
    public enum ZIMGroupEvent
    {
        Created = 1,
        Dismissed = 2,
        Joined = 3,
        Invited = 4,
        Left = 5,
        KickedOut = 6
    };

    // 群状态
    public enum ZIMGroupState { Quit = 0, Enter = 1 };

    public enum ZIMGroupMemberState { Quit = 0, Enter = 1 };

    public enum ZIMGroupMemberEvent
    {
        Joined = 1,
        Left = 2,
        KickedOut = 4,
        Invited = 5
    };

    public enum ZIMCallInvitationMode
    {
        Unknown = -1,
        General = 0,
        Advanced = 1
    };

    public enum ZIMCallState
    {
        Unknown = -1,
        Started = 1,
        Ended = 2
    };

    public static class ZIMGroupMemberRole
    {
        public static int Owner = 1;
        public static int Member = 3;
    }

    public enum ZIMGroupMessageNotificationStatus
    {
        Notify = 1,
        DoNotDisturb = 2
    };

    public enum ZIMCallUserState
    {
        Unknown = -1,
        Inviting = 0,
        Accepted = 1,
        Rejected = 2,
        Cancelled = 3,
        [System.Obsolete("Offline is deprecated since ZIM 2.9.0")]
        Offline = 4,
        Received = 5,
        Timeout = 6,
        Quit = 7
    };

    // User information object.
    //
    // Description: Identifies a unique user.
    //
    // Caution: Note that the userID must be unique under the same appID, otherwise mutual kicks out will occur.
    // It is strongly recommended that userID corresponds to the user ID of the business APP,
    // that is, a userID and a real user are fixed and unique, and should not be passed to the SDK in a random userID.
    // Because the unique and fixed userID allows ZEGO technicians to quickly locate online problems.
    //
    public class ZIMAppConfig
    {
        public uint appID;
        public string appSign;
        public ZIMAppConfig()
        {
            appID = 0;
            appSign = "";
        }
    };

    public class ZIMUserInfo
    {
        // User ID, a string with a maximum length of 32 bytes or less. Only support numbers, English characters and '~', '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '_', '+', '=', '-', '`', ';', '’', ',', '.', '<', '>', '/', '\'.
        public string userID;
        // User name, a string with a maximum length of 64 bytes or less
        public string userName;
        public ZIMUserInfo()
        {
            userID = "";
            userName = "";
        }
        public ZIMUserInfo(string id, string name)
        {
            this.userID = id;
            this.userName = name;
        }
    };

    public class ZIMRoomMemberInfo : ZIMUserInfo
    {
        public ZIMRoomMemberInfo() { }
        public ZIMRoomMemberInfo(string id, string name) : base(id, name) { }
    };

    public class ZIMUsersInfoQueryConfig
    {
        public bool isQueryFromServer;
        public ZIMUsersInfoQueryConfig()
        {
            this.isQueryFromServer = false;
        }
        public ZIMUsersInfoQueryConfig(bool is_query_server)
        {
            this.isQueryFromServer = is_query_server;
        }
    };
    public class ZIMUserFullInfo
    {
        public ZIMUserInfo baseInfo;

        public string userAvatarUrl;

        public string extendedData;

        public ZIMUserFullInfo()
        {
            baseInfo = new ZIMUserInfo();
            userAvatarUrl = "";
            extendedData = "";
        }
        public ZIMUserFullInfo(string id, string name, string avatar_url,
                    string extended_data)
        {
            baseInfo = new ZIMUserInfo();
            this.baseInfo.userID = id;
            this.baseInfo.userName = name;
            this.userAvatarUrl = avatar_url;
            this.extendedData = extended_data;
        }
    };

    public class ZIMErrorUserInfo
    {
        public string userID;
        public uint reason;
        public ZIMErrorUserInfo()
        {
            this.reason = 0;
            userID = "";
        }
        public ZIMErrorUserInfo(string user_id, uint reason)
        {
            this.userID = user_id;
            this.reason = reason;
        }
    };

    // Group information.
    public class ZIMGroupInfo
    {
        public string groupID;
        public string groupName;
        public string groupAvatarUrl;
        public ZIMGroupInfo()
        {
            groupID = "";
            groupName = "";
            groupAvatarUrl = "";
        }
        public ZIMGroupInfo(string group_id, string group_name,
                 string group_avatar_url)
        {
            this.groupID = group_id;
            this.groupName = group_name;
            this.groupAvatarUrl = group_avatar_url;
        }
    };

    // Group advanced configuration.
    public class ZIMGroupAdvancedConfig
    {
        public string groupNotice;
        public Dictionary<string, string> groupAttributes;
        public ZIMGroupAdvancedConfig()
        {
            groupNotice = "";
            groupAttributes = new Dictionary<string, string>();
        }
    };

    // ZIMGroup
    public class ZIMGroup
    {
        public ZIMGroupInfo baseInfo;
        public ZIMGroupMessageNotificationStatus notificationStatus;
        public ZIMGroup()
        {
            baseInfo = new ZIMGroupInfo();
            this.notificationStatus =
                ZIMGroupMessageNotificationStatus.Notify;
        }
    };

    // Complete group information.
    public class ZIMGroupFullInfo
    {
        public ZIMGroupInfo baseInfo;
        public string groupNotice;
        public Dictionary<string, string> groupAttributes;
        public ZIMGroupMessageNotificationStatus notificationStatus;
        public ZIMGroupFullInfo()
        {
            groupNotice = "";
            baseInfo = new ZIMGroupInfo();
            groupAttributes = new Dictionary<string, string>();
            this.notificationStatus =
                ZIMGroupMessageNotificationStatus.Notify;
        }
    };

    // Group member information.
    public class ZIMGroupMemberInfo : ZIMUserInfo
    {
        public string memberNickname;
        public int memberRole;
        public string memberAvatarUrl;
        public ZIMGroupMemberInfo()
        {
            memberNickname = "";
            memberAvatarUrl = "";
            this.memberRole = ZIMGroupMemberRole.Member;
        }
    };

    // Information that the group has operated on.
    public class ZIMGroupOperatedInfo
    {
        public string userID;
        public string userName;
        public string memberNickname;
        public int memberRole;
        public ZIMGroupOperatedInfo()
        {
            userID = "";
            userName = "";
            memberNickname = "";
            this.memberRole = ZIMGroupMemberRole.Member;
        }
    };

    // Group member query configuration.
    public class ZIMGroupMemberQueryConfig
    {
        public uint count;
        public uint nextFlag;
        public ZIMGroupMemberQueryConfig()
        {
            this.count = 0;
            this.nextFlag = 0;
        }
    };

    // MARK: - Model

    // Error structure
    //
    // Description: A structure composed of error code enumeration values and detailed description of the corresponding error information.
    //
    public class ZIMError
    {
        // The storage path of the log files. Refer to the official website document for the default path.
        public ZIMErrorCode code;
        // Error infomation description.
        public string message;
        public ZIMError()
        {
            message = "";
            this.code = ZIMErrorCode.Success;
        }
    };

    // Log configuration
    //
    // Description: Configure the storage path of log files and the maximum log file size.
    //
    public class ZIMLogConfig
    {
        // The storage path of the log files. Refer to the official website document for the default path.
        public string logPath;

        // The maximum log file size (Bytes). The default maximum size is 5MB (5 * 1024 * 1024 Bytes)
        public ulong logSize;
        public ZIMLogConfig()
        {
            logPath = "";
            this.logSize = 5 * 1024 * 1024;
        }
        public ZIMLogConfig(string path, ulong size)
        {
            this.logPath = path;
            this.logSize = size;
        }
    };

    // Cache configuration
    //
    // Description: Configure the storage path of cache files (such as chat records).
    //
    public class ZIMCacheConfig
    {
        // The storage path of the cache files. Refer to the official website document for the default path.
        public string cachePath;
        public ZIMCacheConfig()
        {
            cachePath = "";
        }
    };

    // Base class of message object
    //
    // Description: Identifies the basic parameters of a message.
    //
    // Caution: Some of the parameters, such as Message ID, only have values ​​during the callback.
    // Developers do not need to assign values ​​to these parameters when they actively create this object for sending messages.
    //
    public class ZIMMessage
    {

        // Identifies the type of this message.
        public ZIMMessageType type { get; }
        // Description: The unique ID that identifies this message.
        //
        // Use cases: Can be used to index other messages.
        //
        // Caution: When the developer actively creates a message, there is no need to modify this parameter.
        // This parameter only has a value during callback.
        public long messageID { get; }

        public long localMessageID { get; }

        // Description: Identifies which User sent this message.
        //
        // Use cases: Used to show who sent a message.
        //
        // Caution: When the developer actively creates a message, there is no need to modify this parameter.
        // This parameter only has a value during callback.
        public string senderUserID { get; }

        public string conversationID { get; }

        public ZIMConversationType conversationType { get; }

        public long conversationSeq { get; }

        public ZIMMessageDirection direction { get; }

        public ZIMMessageSentStatus sentStatus { get; }

        public ZIMMessageReceiptStatus receiptStatus { get; }

        // Description: Identifies the sending time of a message
        //
        // Use cases: Used to present the sending time of a message, and can be used for message sorting.
        //
        // Caution: This is a standard UNIX timestamp, in milliseconds.
        public ulong timestamp { get; }

        public long orderKey { get; }

        public bool isUserInserted { get; }

        public bool isBroadcastMessage { get; }

        public string extendedData { get; set; }

        public string localExtendedData { get; set; }

        public List<ZIMMessageReaction> reactions { get; set; }
        public ZIMMessage() 
        {
            senderUserID = "";
            conversationID = "";
            extendedData = "";
            localExtendedData = "";
            reactions = new List<ZIMMessageReaction>();
        }
        public ZIMMessage(ZIMMessageType type)
        {
            this.type = type;
            senderUserID = "";
            conversationID = "";
            extendedData = "";
            localExtendedData = "";
            reactions = new List<ZIMMessageReaction>();
        }
    }

    // Description:Offline push configuration.
    public class ZIMPushConfig
    {
        public string title;
        public string content;
        public string payload;
        public string resourcesID;
        public ZIMPushConfig()
        {
            title = "";
            content = "";
            payload = "";
            resourcesID = "";
        }
    };

    // Configure message sending.
    public class ZIMMessageSendConfig
    {
        public ZIMPushConfig pushConfig;
        public ZIMMessagePriority priority;
        public bool hasReceipt;
        public ZIMMessageSendConfig()
        {
            pushConfig = null;
            priority = ZIMMessagePriority.Low;
            hasReceipt = false;
        }
    };

    // Example Query message configuration.
    public class ZIMMessageQueryConfig
    {
        public ZIMMessage nextMessage;
        public uint count;
        public bool reverse;
        public ZIMMessageQueryConfig()
        {
            nextMessage = null;
            count = 0;
            reverse = false;
        }
    };

    // Delete configurations related to messages.
    public class ZIMMessageDeleteConfig
    {
        public bool isAlsoDeleteServerMessage;
        public ZIMMessageDeleteConfig()
        {
            this.isAlsoDeleteServerMessage = true;
        }
    };

    public class ZIMMessageRevokeConfig
    {
        public ZIMPushConfig config;
        public string revokeExtendedData;
        public ZIMMessageRevokeConfig()
        {
            config = null;
            revokeExtendedData = "";
        }
    };


    public class ZIMMessageSearchConfig
    {
        public ZIMMessage nextMessage;
        public uint count;
        public ZIMMessageOrder order;
        public List<string> keywords;
        public List<ZIMMessageType> messageTypes;
        public List<uint> subMessageTypes;
        public List<string> senderUserIDs;
        public long startTime;
        public long endTime;

        public ZIMMessageSearchConfig()
        {
            nextMessage = null;
            count = 0;
            order = ZIMMessageOrder.Descending;
            keywords = new List<string>();
            messageTypes = new List<ZIMMessageType>();
            subMessageTypes = new List<uint>();
            senderUserIDs = new List<string>();
        }
    }

    public class ZIMMessageReactionUserQueryConfig
    {
        public ulong nextFlag;
        public uint count;
        public string reactionType;

        public ZIMMessageReactionUserQueryConfig()
        {
            nextFlag = 0;
            count = 0;
            reactionType = "";
        }
    };

    public class ZIMMessageReactionUserInfo
    {
        public string userID;

        public ZIMMessageReactionUserInfo()
        {
            userID = "";
        }
    };

    public class ZIMMessageReaction
    {
        public string reactionType;
        public string conversationID;
        public ZIMConversationType conversationType;
        public long messageID;
        public uint totalCount;
        public bool isSelfIncluded;
        public List<ZIMMessageReactionUserInfo> userList;

        public ZIMMessageReaction()
        {
            reactionType = "";
            conversationID = "";
            userList = new List<ZIMMessageReactionUserInfo>();
        }
    };

    // Session specific content.
    public class ZIMConversation
    {
        public string conversationID;
        public string conversationName;
        public string conversationAvatarUrl;
        public ZIMConversationType type;
        public uint unreadMessageCount;
        public ZIMMessage lastMessage = null;
        public long orderKey;
        public ZIMConversationNotificationStatus notificationStatus;
        public bool isPinned;
        public ZIMConversation()
        {
            conversationID = "";
            conversationName = "";
            conversationAvatarUrl = "";
            type = ZIMConversationType.Peer;
            unreadMessageCount = 0;
            orderKey = -1;
            notificationStatus = ZIMConversationNotificationStatus.Notify;
            isPinned = false;
        }
    };

    public class ZIMMessageSentStatusChangeInfo
    {
        public ZIMMessageSentStatus status;
        public ZIMMessage message;
        public ZIMMessageSentStatusChangeInfo()
        {
            message = null;
        }
    };

    //  Session change information.
    public class ZIMConversationChangeInfo
    {
        public ZIMConversationEvent conversationEvent;
        public ZIMConversation conversation;
        public ZIMConversationChangeInfo()
        {
            conversation = null;
        }
    };

    // Query the relevant configuration of the session.
    public class ZIMConversationQueryConfig
    {
        public ZIMConversation nextConversation;
        public int count;
        public ZIMConversationQueryConfig()
        {
            nextConversation = null;
        }
    };

    public class ZIMConversationSearchConfig
    {
        public uint nextFlag;
        public uint totalConversationCount;
        public uint conversationMessageCount;
        public List<string> keywords;
        public List<ZIMMessageType> messageTypes;
        public List<uint> subMessageTypes;
        public List<string> senderUserIDs;
        public long startTime;
        public long endTime;

        public ZIMConversationSearchConfig()
        {
            keywords = new List<string>();
            messageTypes = new List<ZIMMessageType>();
            subMessageTypes = new List<uint>();
            senderUserIDs = new List<string>();
        }
    }

    public class ZIMConversationSearchInfo
    {
        public string conversationID;
        public ZIMConversationType conversationType;
        public uint totalMessageCount;
        public List<ZIMMessage> messageList;

        public ZIMConversationSearchInfo()
        {
            conversationID = "";
            messageList = new List<ZIMMessage>();
        }
    };

    // Delete session configuration.
    public class ZIMConversationDeleteConfig
    {
        public bool isAlsoDeleteServerConversation;
        public ZIMConversationDeleteConfig()
        {
            this.isAlsoDeleteServerConversation = true;
        }
    };

    // Normal text message object.
    //
    // Description: Identifies the basic parameters of a message.
    //
    // Caution: If the Type parameter of the base class is Text during callback,
    // you can force the base class message object to be of this type.
    //
    public class ZIMTextMessage : ZIMMessage
    {

        // The content of the text message.
        public string message;

        public ZIMTextMessage() : base(ZIMMessageType.Text)
        {
            message = "";
        }
        public ZIMTextMessage(string msg) : base(ZIMMessageType.Text)
        {
            this.message = msg;
        }
    };

    public class ZIMMediaMessage : ZIMMessage
    {
        public string fileLocalPath;
        public string fileDownloadUrl;

        public string fileUID { get; }
        public string fileName { get; set; }
        public ulong fileSize { get; }
        public ZIMMediaMessage(ZIMMessageType mediaType) : base(mediaType)
        {
            fileLocalPath = "";
            fileDownloadUrl = "";
            fileUID = "";
            fileName = "";
        }
        public ZIMMediaMessage(ZIMMessageType mediaType,
            string localPath, string fileUrl) : base(mediaType)
        {
            fileLocalPath = localPath;
            fileDownloadUrl = fileUrl;
            fileUID = "";
            fileName = "";
        }
    }
    public class ZIMImageMessage : ZIMMediaMessage
    {
        public string thumbnailDownloadUrl;
        public string largeImageDownloadUrl;

        public string thumbnailLocalPath { get; }
        public string largeImageLocalPath { get; }
        public uint originalImageWidth { get; }
        public uint originalImageHeight { get; }
        public uint largeImageWidth { get; }
        public uint largeImageHeight { get; }
        public uint thumbnailWidth { get; }
        public uint thumbnailHeight { get; }

        public ZIMImageMessage(string localPath) :
            base(ZIMMessageType.Image, localPath, "")
        {
            thumbnailDownloadUrl = "";
            largeImageDownloadUrl = "";
            thumbnailLocalPath = "";
            largeImageLocalPath = "";
        }
        public ZIMImageMessage(string fileUrl, string largeUrl, string thumbnailUrl) :
            base(ZIMMessageType.Image, "", fileUrl)
        {
            thumbnailDownloadUrl = thumbnailUrl;
            largeImageDownloadUrl = largeUrl;
            thumbnailLocalPath = "";
            largeImageLocalPath = "";
        }

    }
    public class ZIMFileMessage : ZIMMediaMessage
    {
        public ZIMFileMessage(string localPath)
              : base(ZIMMessageType.File, localPath, "") { }
    };

    public class ZIMAudioMessage : ZIMMediaMessage
    {
        public uint audioDuration;
        public ZIMAudioMessage(string localPath, uint duration)
            : base(ZIMMessageType.Audio, localPath, "")
        {
            this.audioDuration = duration;
        }
    };

    public class ZIMVideoMessage : ZIMMediaMessage
    {
        public uint videoDuration;
        public string videoFirstFrameDownloadUrl;

        public string videoFirstFrameLocalPath { get; }
        public uint videoFirstFrameWidth { get; }
        public uint videoFirstFrameHeight { get; }
        public ZIMVideoMessage(string videoPath, uint duration)
        : base(ZIMMessageType.Video, videoPath, "")
        {
            videoDuration = duration;
            videoFirstFrameDownloadUrl = "";
            videoFirstFrameLocalPath = "";
        }
        public ZIMVideoMessage(string videoUrl, string videoFirstFrameUrl,
                    uint duration)
        : base(ZIMMessageType.Video, "", videoUrl)
        {
            videoDuration = duration;
            videoFirstFrameDownloadUrl = videoFirstFrameUrl;
            videoFirstFrameLocalPath = "";
        }
    };

    [System.Obsolete("Deprecated since ZIM 2.8.0, please use [ZIMCustomMessage] instead.")]
    public class ZIMSystemMessage : ZIMMessage
    {
        // The content of the text message.
        public string message;
        public ZIMSystemMessage() : base(ZIMMessageType.System)
        {
            message = "";
        }
        public ZIMSystemMessage(string msg) : base(ZIMMessageType.System)
        {
            this.message = msg;
        }
    };

    public class ZIMCustomMessage : ZIMMessage
    {
        // The content of the text message.
        public string message;
        public uint subType = 0;
        public string searchedContent;

        public ZIMCustomMessage(string msg, uint subType)
            : base(ZIMMessageType.Custom)
        {
            message = msg;
            this.subType = subType;
            searchedContent = "";
        }
    };

    public class ZIMRevokeMessage : ZIMMessage
    {
        public ZIMRevokeType revokeType { get; }
        public uint revokeTimestamp { get; }
        public string operatedUserID { get; }
        public ZIMMessageType originalMessageType { get; }
        public string originalTextMessageContent { get; }
        public string revokeExtendedData { get; }
        public ZIMMessageRevokeStatus revokeStatus { get; }
        public ZIMRevokeMessage() : base(ZIMMessageType.Revoke)
        {
            revokeType = ZIMRevokeType.Unknown;
            originalMessageType = ZIMMessageType.Unknown;
            revokeStatus = ZIMMessageRevokeStatus.Unknown;
            operatedUserID = "";
            originalTextMessageContent = "";
            revokeExtendedData = "";
        }
    };

    // Custom binary message object.
    //
    // Description: Identifies a binary message.
    //
    // Caution: If the Type parameter of the base class is Custom during callback,
    // you can force the base class message object to be of this type.
    //
    public class ZIMCommandMessage : ZIMMessage
    {
        // The content of the custom message.
        public byte[] message;
        public ZIMCommandMessage() : base(ZIMMessageType.Command) { }
        public ZIMCommandMessage(byte[] msg)
        : base(ZIMMessageType.Command)
        {
            this.message = msg;
        }
    };

    // Barrage text message object.
    //
    // Description: This message type only supports sending in the room, and is used in scenarios where there is a large concurrency that does not require delivery.
    //
    //
    public class ZIMBarrageMessage : ZIMMessage
    {
        // The content of the text message.
        public string message;
        public ZIMBarrageMessage() : base(ZIMMessageType.Barrage)
        {
            message = "";
        }
        public ZIMBarrageMessage(string msg)
            : base(ZIMMessageType.Barrage)
        {
            this.message = msg;
        }
    };

    public delegate void ZIMMessageAttachedCallback(ZIMMessage message);

    public delegate void ZIMMediaUploadingProgress(ZIMMessage message,
        ulong currentFileSize, ulong totalFileSize);

    public class ZIMMessageSendNotification
    {
        public ZIMMessageAttachedCallback onMessageAttached;

        public ZIMMessageSendNotification()
        {
            onMessageAttached = null;
        }
        public ZIMMessageSendNotification(ZIMMessageAttachedCallback cbAttach)
        {
            this.onMessageAttached = cbAttach;
        }
    };

    public class ZIMMediaMessageSendNotification
    {
        public ZIMMessageAttachedCallback onMessageAttached;
        public ZIMMediaUploadingProgress onMediaUploadingProgress;

        public ZIMMediaMessageSendNotification()
        {
            onMessageAttached = null;
            onMediaUploadingProgress = null;
        }
        public ZIMMediaMessageSendNotification(ZIMMessageAttachedCallback cbAttach,
                                        ZIMMediaUploadingProgress cbProgress)
        {
            this.onMessageAttached = cbAttach;
            this.onMediaUploadingProgress = cbProgress;
        }
    };

    public class ZIMMessageReceiptInfo
    {
        public ZIMMessageReceiptStatus status;
        public long messageID;
        public string conversationID;
        public ZIMConversationType conversationType;
        public uint readMemberCount;
        public uint unreadMemberCount;
        public ZIMMessageReceiptInfo()
        {
            conversationID = "";
        }
    };

    // Room information object.
    //
    // Description: Identifies a unique room.
    //
    public class ZIMRoomInfo
    {
        // Room ID, a string with a maximum length of 32 bytes or less. Only support numbers, English characters and '~', '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '_', '+', '=', '-', '`', ';', '’', ',', '.', '<', '>', '/', '\'.
        public string roomID;

        // Room name, a string with a maximum length of 64 bytes or less
        public string roomName;
        public ZIMRoomInfo()
        {
            roomID = "";
            roomName = "";
        }
        public ZIMRoomInfo(string id, string name)
        {
            this.roomID = id;
            this.roomName = name;
        }
    };

    // The room details object.
    //
    // Description: Identifies the basic parameters of a message.
    //
    // Caution: Identifies the detailed information of a room.
    //
    public class ZIMRoomFullInfo
    {
        // The base object of the room.
        public ZIMRoomInfo baseInfo;

        public ZIMRoomFullInfo()
        {
            baseInfo = new ZIMRoomInfo();
        }
    };


    // Configuration for querying member.
    //
    // Description: When querying member, you need to configure this object.
    //
    public class ZIMRoomMemberQueryConfig
    {
        // Description: The flag of the paging query. For the first query, set this field to an empty string. If the "nextFlag" field of the callback is not an empty string, it needs to be set here to continue the query on the next page.
        public string nextFlag;

        // Description: How many messages are retrieved in one query.
        //
        // Caution: To obtain messages in pages to reduce overhead, it is recommended to obtain within 100 messages at a time.
        public uint count;

        public ZIMRoomMemberQueryConfig()
        {
            nextFlag = "";
        }
        ZIMRoomMemberQueryConfig(string flag, uint cnt)
        {
            this.nextFlag = flag;
            this.count = cnt;
        }
    };

    // Configuration of room advanced
    //
    // Description: When create a room, user could add roomAttributes and destroy delay time by using this config
    //
    public class ZIMRoomAdvancedConfig
    {
        // Description: Room attributes of a room.
        public Dictionary<string, string> roomAttributes;
        // Description: Room will exist time after all of the users in the room left
        public uint roomDestroyDelayTime = 0;

        public ZIMRoomAdvancedConfig()
        {
            roomAttributes = new Dictionary<string, string>();
        }
    };


    // The behavior attribute set by the room attribute.
    //
    // Description: The behavior attribute set by the room attribute.
    //
    public class ZIMRoomAttributesSetConfig
    {
        // Description: Room attributes are automatically deleted after the owner leaves the room.
        public bool isDeleteAfterOwnerLeft;
        // Description: Whether the operation is mandatory, that is, the property of the room whose owner is another user can be modified.
        public bool isForce;
        // Description: Whether to update the owner of the room attribute involved.
        public bool isUpdateOwner;

        public ZIMRoomAttributesSetConfig()
        {
            this.isDeleteAfterOwnerLeft = false;
            this.isForce = false;
            this.isUpdateOwner = false;
        }
        ZIMRoomAttributesSetConfig(bool left, bool force, bool update)
        {
            this.isDeleteAfterOwnerLeft = left;
            this.isForce = force;
            this.isUpdateOwner = update;
        }
    };

    // The behavior attribute set by the room attribute.
    //
    // Description: The behavior attribute set by the room attribute.
    //
    public class ZIMRoomAttributesDeleteConfig
    {
        // Description: Whether the operation is mandatory, that is, the property of the room whose owner is another user can be deleted.
        public bool isForce;

        public ZIMRoomAttributesDeleteConfig() { this.isForce = false; }
        public ZIMRoomAttributesDeleteConfig(bool force) { this.isForce = force; }
    };

    // The behavior attribute set by the room attribute.
    //
    // Description: The behavior attribute set by the room attribute.
    //
    public class ZIMRoomAttributesBatchOperationConfig
    {
        // Description: Room attributes are automatically deleted after the owner leaves the room.
        public bool isDeleteAfterOwnerLeft;
        // Description: Whether the operation is mandatory, that is, the property of the room whose owner is another user can be modified.
        public bool isForce;
        // Description: Whether to update the owner of the room attribute involved.
        public bool isUpdateOwner;

        public ZIMRoomAttributesBatchOperationConfig()
        {
            this.isDeleteAfterOwnerLeft = false;
            this.isForce = false;
            this.isUpdateOwner = false;
        }
        ZIMRoomAttributesBatchOperationConfig(bool left, bool force, bool update)
        {
            this.isDeleteAfterOwnerLeft = left;
            this.isForce = force;
            this.isUpdateOwner = update;
        }
    };

    // Notice of Room Attribute Change.
    //
    // Description: Notice of Room Attribute Change.
    //
    public class ZIMRoomAttributesUpdateInfo
    {
        // Description: Behavioral information of room attribute change notification.
        public ZIMRoomAttributesUpdateAction action;
        // Description: Room attributes.
        public Dictionary<string, string> roomAttributes;
        public ZIMRoomAttributesUpdateInfo()
        {
            roomAttributes = new Dictionary<string, string>();
        }
    };

    // Room user attribute information.
    public class ZIMRoomMemberAttributesInfo
    {
        public ZIMRoomMemberAttributesInfo()
        {
            userID = "";
            attributes = new Dictionary<string, string>();
        }

        public string userID;

        //Detailed description: room user attributes.
        public Dictionary<string, string> attributes;
    };

    // Room user attribute operation information.
    public class ZIMRoomMemberAttributesOperatedInfo
    {
        public ZIMRoomMemberAttributesOperatedInfo()
        {
            attributesInfo = new ZIMRoomMemberAttributesInfo();
            errorKeys = new List<string>();
        }
        //Detailed description: room user attribute information.
        public ZIMRoomMemberAttributesInfo attributesInfo;

        //Detail description: The key of the room user attribute operation failure.
        public List<string> errorKeys;
    };

    // Detail description: room user attribute setting configuration, including configurable whether to retain user attributes after the user leaves, etc.
    // Business scenarios: When setting room user attributes, developers can choose to use them according to their actual application scenarios.
    public class ZIMRoomMemberAttributesSetConfig
    {
        // Detail description: Configure whether the room user attributes are saved after the user leaves the room.

        // Required: not required.

        // Default: true.
        public bool isDeleteAfterOwnerLeft;
        public ZIMRoomMemberAttributesSetConfig() { this.isDeleteAfterOwnerLeft = true; }
    };

    // Room user attribute update information.
    public class ZIMRoomMemberAttributesUpdateInfo
    {
        public ZIMRoomMemberAttributesUpdateInfo()
        {
            attributesInfo = new ZIMRoomMemberAttributesInfo();
        }
        // Detailed description: room user attributes.
        public ZIMRoomMemberAttributesInfo attributesInfo;
    };

    // Detailed description: Room operation information.
    public class ZIMRoomOperatedInfo
    {
        //Detail description: UserID of the operator.
        public string userID;
        public ZIMRoomOperatedInfo()
        {
            userID = "";
        }
    };

    // Room user attribute query configuration.
    public class ZIMRoomMemberAttributesQueryConfig
    {
        // Detail description: Query anchor for room user properties.

        // Required or not: not required, it is empty by default for the first time, which means to start the query from the beginning.
        public string nextFlag;

        // Detail description: The number of paginated queries.

        // Required: Required.
        public uint count;

        public ZIMRoomMemberAttributesQueryConfig()
        {
            nextFlag = "";
        }
        public ZIMRoomMemberAttributesQueryConfig(string flag, uint cnt)
        {
            this.nextFlag = flag;
            this.count = cnt;
        }
    };

    // Group attribute update information.
    //
    // Description: Group attribute update information.
    //
    public class ZIMGroupAttributesUpdateInfo
    {
        // Description: Group attribute update action.
        public ZIMGroupAttributesUpdateAction action;

        // Description: group attributes.
        public Dictionary<string, string> groupAttributes;

        public ZIMGroupAttributesUpdateInfo()
        {
            groupAttributes = new Dictionary<string, string>();
        }
    };

    public class ZIMGroupMessageReceiptMemberQueryConfig
    {
        public uint nextFlag;
        public uint count;

        public ZIMGroupMessageReceiptMemberQueryConfig() { }
    };

    public class ZIMGroupSearchConfig
    {
        public uint nextFlag;
        public uint count;
        public List<string> keywords;
        public bool isAlsoMatchGroupMemberUserName;
        public bool isAlsoMatchGroupMemberNickname;

        public ZIMGroupSearchConfig()
        {
            keywords = new List<string>();
        }
    };

    public class ZIMGroupMemberSearchConfig
    {
        public uint nextFlag;
        public uint count;
        public List<string> keywords;
        public bool isAlsoMatchGroupMemberNickname;

        public ZIMGroupMemberSearchConfig()
        {
            keywords = new List<string>();
        }
    };

    public class ZIMGroupSearchInfo
    {
        public ZIMGroupInfo groupInfo;
        public List<ZIMGroupMemberInfo> userList;

        public ZIMGroupSearchInfo()
        {
            groupInfo = new ZIMGroupInfo();
            userList = new List<ZIMGroupMemberInfo>();
        }
    };

    // MARK: CallInvite

    // Call invitation user information.
    public class ZIMCallUserInfo
    {
        // Description: userID.
        public string userID;
        // Description: user status.
        public ZIMCallUserState state;
        public string extendedData;

        [System.Obsolete("userState is deprecated after 2.9. Please use state.")]
        public ZIMCallUserState userState;
        public ZIMCallUserInfo()
        {
            userID = "";
            extendedData = "";
        }
    };

    // The behavior property of the Send Call Invitation setting.
    public class ZIMCallInviteConfig
    {
        // Description: The timeout setting of the call invitation, the unit is seconds.
        public int timeout;
        // Description: Extended field, through which the inviter can carry information to the invitee.
        public string extendedData;
        public ZIMCallInvitationMode mode;
        public ZIMPushConfig pushConfig = null;

        public ZIMCallInviteConfig()
        {
            timeout = 90;
            extendedData = "";
        }
    };

    public class ZIMCallingInviteConfig
    {
        public ZIMPushConfig pushConfig = null;

        public ZIMCallingInviteConfig() { }
    };

    public class ZIMCallInfo
    {
        public string callID;
        public string caller;
        public ZIMCallInvitationMode mode;
        public ZIMCallState state;
        // Description: Extended field, through which the inviter can carry information to the invitee.
        public string extendedData;
        public long createTime;
        public long endTime;
        public List<ZIMCallUserInfo> callUserList;

        public ZIMCallInfo()
        {
            callID = "";
            caller = "";
            extendedData = "";
            callUserList = new List<ZIMCallUserInfo>();
        }
    };

    public class ZIMCallCancelConfig
    {
        // Description: Extended field, through which the inviter can carry information to the invitee.
        public string extendedData;
        public ZIMCallCancelConfig()
        {
            extendedData = "";
        }
    };

    public class ZIMCallInvitationEndedInfo
    {
        public string caller;
        public string operatedUserID;
        public string extendedData;
        public ZIMCallInvitationMode mode;
        public long endTime;
        public ZIMCallInvitationEndedInfo()
        {
            caller = "";
            operatedUserID = "";
            extendedData = "";
        }
    };

    public class ZIMCallUserStateChangeInfo
    {
        public List<ZIMCallUserInfo> callUserList;

        public ZIMCallUserStateChangeInfo()
        {
            callUserList = new List<ZIMCallUserInfo>();
        }
    };

    // Behavior property set by accepting a call invitation.
    public class ZIMCallAcceptConfig
    {
        // Description: Extended field, through which the inviter can carry information to the invitee.
        public string extendedData;
        public ZIMCallAcceptConfig()
        {
            extendedData = "";
        }
    };

    // The behavior property of the reject call invitation setting.
    public class ZIMCallRejectConfig
    {
        // Description: Extended field, through which the inviter can carry information to the invitee.
        public string extendedData;
        public ZIMCallRejectConfig()
        {
            extendedData = "";
        }
    };

    public class ZIMCallQuitConfig
    {
        // Description: Extended field, through which the inviter can carry information to the invitee.
        public string extendedData;

        public ZIMPushConfig pushConfig;
        public ZIMCallQuitConfig()
        {
            extendedData = "";
            pushConfig = null;
        }
    };

    public class ZIMCallEndConfig
    {
        // Description: Extended field, through which the inviter can carry information to the invitee.
        public string extendedData;

        public ZIMPushConfig pushConfig;
        public ZIMCallEndConfig()
        {
            extendedData = "";
            pushConfig = null;
        }
    };

    public class ZIMCallInvitationQueryConfig
    {
        public uint count;
        public long nextFlag;

        public ZIMCallInvitationQueryConfig() { }
    };

    // Call invitation sent message.
    public class ZIMCallInvitationSentInfo
    {
        // Description: The timeout setting of the call invitation, the unit is seconds.
        public int timeout;
        // Description: User id that has not received a call invitation.
        [System.Obsolete("Deprecated since 2.9.0, pleace use  [errorUserList] instead")]
        public List<ZIMCallUserInfo> errorInvitees;

        public List<ZIMCallUserInfo> errorUserList;

        public ZIMCallInvitationSentInfo()
        {
#pragma warning disable CS0618
            errorInvitees = new List<ZIMCallUserInfo>();
#pragma warning restore CS0618
            errorUserList = new List<ZIMCallUserInfo>();
        }
    };

    public class ZIMCallingInvitationSentInfo
    {
        // Description: User id that has not received a call invitation.
        public List<ZIMErrorUserInfo> errorUserList;

        public ZIMCallingInvitationSentInfo()
        {
            errorUserList = new List<ZIMErrorUserInfo>();
        }
    };

    // Information to accept the call invitation.
    public class ZIMCallInvitationReceivedInfo
    {
        // Description: The timeout setting of the call invitation, the unit is seconds.
        public int timeout;
        // Description: Inviter ID.
        public string inviter;
        // Description: Extended field, through which the inviter can carry information to the invitee.
        public string extendedData;
        public ZIMCallInvitationReceivedInfo()
        {
            inviter = "";
            extendedData = "";
        }
    };

    // Cancel the call invitation message.
    public class ZIMCallInvitationCancelledInfo
    {
        // Description: The inviter ID of the call invitation.
        public string inviter;
        // Description: Extended field, through which the inviter can carry information to the invitee.
        public string extendedData;
        public ZIMCallInvitationCancelledInfo()
        {
            inviter = "";
            extendedData = "";
        }
    };

    public class ZIMCallQuitSentInfo
    {
        public long createTime;
        public long acceptTime;
        public long quitTime;

        public ZIMCallQuitSentInfo() { }
    };

    public class ZIMCallEndedSentInfo
    {
        public long createTime;
        public long acceptTime;
        public long endTime;

        public ZIMCallEndedSentInfo() { }
    };

    // Accept the call invitation message.
    public class ZIMCallInvitationAcceptedInfo
    {
        // Description: Invitee ID.
        public string invitee;
        // Description: Extended field, through which the inviter can carry information to the invitee.
        public string extendedData;
        public ZIMCallInvitationAcceptedInfo()
        {
            invitee = "";
            extendedData = "";
        }
    };

    // Reject the call invitation message.
    public class ZIMCallInvitationRejectedInfo
    {
        // Description: Invitee ID.
        public string invitee;
        // Description: Extended field, through which the inviter can carry information to the invitee.
        public string extendedData;
        public ZIMCallInvitationRejectedInfo()
        {
            invitee = "";
            extendedData = "";
        }
    };

    public class ZIMCallInvitationTimeoutInfo
    {
        public ZIMCallInvitationMode mode;

        public ZIMCallInvitationTimeoutInfo() { }
    };

    // MARK: Main
    //
    // Callback of the log upload result .
    //
    // @param errorCode Error code.
    public delegate void ZIMLogUploadedCallback(ZIMError errorInfo);

    // Callback of the result login.
    //
    // @param errorCode Error code.
    public delegate void ZIMLoggedInCallback(ZIMError errorInfo);

    // Callback of the result of renewing the token.
    //
    // @param token The renewed token.
    // @param errorCode Error code.
    public delegate void ZIMTokenRenewedCallback(string token, ZIMError errorInfo);

    public delegate void ZIMUserNameUpdatedCallback(string userName, ZIMError errorInfo);

    public delegate void ZIMUserAvatarUrlUpdatedCallback(string userAvatarUrl, ZIMError errorInfo);

    public delegate void ZIMUserExtendedDataUpdatedCallback(string extendedData, ZIMError errorInfo);

    public delegate void ZIMUsersInfoQueriedCallback(List<ZIMUserFullInfo> userFullInfos,
                                List<ZIMErrorUserInfo> errorUserList, ZIMError errorInfo);

    // MARK: Message

    // Callback of the result of sending the message.
    //
    // @param message The sent message object, from which parameters such as messageID can be obtained. If the sending fails, the messageID parameter in the message will be an empty string.
    // @param errorCode Error code.
    public delegate void ZIMMessageSentCallback(ZIMMessage message, ZIMError errorInfo);

    // Callback of the result of querying messages.
    //
    // @param conversationID ConversationID.
    // @param conversationType Conversation Type.
    // @param messageList The message list of the query result.
    // @param errorCode Error code.
    public delegate void ZIMMessageQueriedCallback(string conversationID, ZIMConversationType conversationType,
        List<ZIMMessage> messageList, ZIMError errorInfo);

    public delegate void ZIMMediaDownloadedCallback(ZIMMediaMessage message, ZIMError errorInfo);

    public delegate void ZIMMediaDownloadingProgress(ZIMMediaMessage message,
                           ulong currentFileSize, ulong totalFileSize);

    // A callback to delete a message.
    //
    // @param conversationID ConversationID.
    // @param conversationType Conversation Type.
    // @param errorInfo Delete the error code of the message. 0 means the deletion of the message succeeded, non-0 means the deletion of the message failed. If the deletion message fails, the developer should check the official website error code document to query the solution.

    public delegate void ZIMMessageDeletedCallback(string conversationID, ZIMConversationType conversationType,
                           ZIMError errorInfo);
    public delegate void ZIMMessageRevokedCallback(ZIMMessage message, ZIMError errorInfo);

    public delegate void ZIMMessageReceiptsReadSentCallback(string conversationID, ZIMConversationType conversationType,
                           List<long> errorMessageIDs, ZIMError errorInfo);

    public delegate void ZIMMessageReceiptsInfoQueriedCallback(List<ZIMMessageReceiptInfo> infos, List<long> errorMessageIDs,
                           ZIMError errorInfo);

    public delegate void ZIMMessageLocalExtendedDataUpdatedCallback(ZIMMessage message, ZIMError errorInfo);

    public delegate void ZIMMessagesSearchedCallback(string conversationID, ZIMConversationType conversationType,
                       List<ZIMMessage> messageList,
                       ZIMMessage nextMessage, ZIMError errorInfo);

    public delegate void ZIMMessagesGlobalSearchedCallback(List<ZIMMessage> messageList,
                       ZIMMessage nextMessage, ZIMError errorInfo);

    public delegate void ZIMMessageReactionAddedCallback(ZIMMessageReaction reaction, ZIMError errorInfo);

    public delegate void ZIMMessageReactionDeletedCallback(ZIMMessageReaction reaction, ZIMError errorInfo);

    public delegate void ZIMMessageReactionUserListQueriedCallback(
    ZIMMessage message,
    List<ZIMMessageReactionUserInfo> userList, string reactionType,
    long nextFlag, uint totalCount, ZIMError errorInfo);

    // Supported Versions: 2.4.0 and above.

    // Detail description: A callback for the result of inserting a local message.

    // Business scenario: The developer can use the [errorCode] in the callback to determine whether the insertion is successful.

    // Notification timing: This callback is triggered when the developer calls the [insertMessageToLocalDB] interface.

    // Related interfaces: Insert local messages through [insertMessageToLocalDB], and the success or failure will be notified through this callback.
    // @param Message object.
    // @param errorInfo Insert the error code for the local message. 0 means inserting the message successfully, non-0 means inserting the message failed. If there is a failure to insert the message, the developer should check the official website error code document to query the solution.
    public delegate void ZIMMessageInsertedCallback(ZIMMessage message, ZIMError errorInfo);

    // MARK: Conversation

    public delegate void ZIMConversationQueriedCallback(ZIMConversation conversation,
                       ZIMError errorInfo);

    // Available since: 2.0.0 and above.

    // Description: After the session list is queried, the callback is used to return the query result.

    // Use cases: The logic after the session list query can be done in this callback.

    // When to call /Trigger: Description Triggered when the session list is queried.

    // Related APIs: [queryConversationList].
    // @param conversationID ConversationID.
    // @param conversationType Conversation Type.
    // @param errorInfo Error code for querying the conversation list. 0 means querying the session list succeeds, non-0 means querying the session list fails. If there is a failure to query the call list, the developer should check the official website error code document to query the solution.sage failed. If the deletion message fails, the developer should check the official website error code document to query the solution.

    public delegate void ZIMConversationListQueriedCallback(List<ZIMConversation> conversationList,
                           ZIMError errorInfo);

    public delegate void ZIMConversationPinnedListQueriedCallback(List<ZIMConversation> conversationList,
                       ZIMError errorInfo);

    public delegate void ZIMConversationPinnedStateUpdatedCallback(string conversationID, ZIMConversationType conversationType,
                       ZIMError errorInfo);

    public delegate void ZIMConversationsSearchedCallback(List<ZIMConversationSearchInfo> conversationSearchInfoList,
                       uint nextFlag, ZIMError errorInfo);


    // Callback for conversation deletion.
    //
    // Available since: 2.0.0 and above.
    // Description: After a session is deleted, the deletion result is returned using this callback.
    // Use cases: You can do the deleted session logic in this callback.
    // When to call /Trigger: Description Triggered after the session was deleted.
    // Related APIs: [[deleteConversation].
    // @param conversationID ConversationID.
    // @param conversationType Conversation Type.
    // @param errorInfo Delete the error code of the call. 0 means that the session was deleted successfully, non-0 means that the session was deleted failed. If the deletion of the call fails, the developer should check the official website error code document to query the solution.

    public delegate void ZIMConversationDeletedCallback(string conversationID, ZIMConversationType conversationType,
                           ZIMError errorInfo);

    // Callback for conversation unread clearing.
    //
    // Available since: 2.0.0 and above.
    // Description: This callback returns the result of clearing a session if the session is not read.
    // Use cases: You can do clear unread logic in this callback.
    // When to call /Trigger: Triggered after clearing session unread.
    // Related APIs: [clearConversationUnreadMessageCount].
    // @param conversationID ConversationID.
    // @param conversationType Conversation Type.
    // @param errorInfo Clears the error code for the number of unread calls. 0 means clearing the unread number of the session successfully, non-0 means the clearing the unread number of the session failed. If there is a failure to clear the number of unread calls, the developer should check the official website error code document to query the solution.

    public delegate void ZIMConversationUnreadMessageCountClearedCallback(string conversationID, ZIMConversationType conversationType,
                            ZIMError errorInfo);

    // Callback for conversation notification state settings.
    //
    // Available since: 2.0.0 and above.
    // Description: This callback returns the result of group notification after the group notification status is set.
    // Use cases: You can do the logic after setting the group notification status in this callback.
    // When to call /Trigger: Triggered when the group notification status is set.
    // Related APIs: [setConversationNotificationStatus].
    // @param conversationID ConversationID.
    // @param conversationType Conversation Type.
    // @param errorInfo Set the error code for the callback notification state. 0 means setting the call back notification state successfully, non-0 means setting the call back notification state fails. If there is a failure to set the callback notification status, the developer should check the official website error code document to query the solution.

    public delegate void ZIMConversationNotificationStatusSetCallback(string conversationID, ZIMConversationType conversationType,
                           ZIMError errorInfo);

    public delegate void ZIMConversationMessageReceiptReadSentCallback(string conversationID, ZIMConversationType conversationType,
                           ZIMError errorInfo);

    // MARK: Room

    // Callback of the result of creating the room.
    //
    // @param roomInfo Details of the room created. If the creation fails, the roomID parameter in roomInfo will be an empty string.
    // @param errorCode Error code.
    public delegate void ZIMRoomCreatedCallback(ZIMRoomFullInfo roomInfo, ZIMError errorInfo);

    // Callback of the result of joining the room.
    //
    // @param roomInfo Details of the room joined. If the join fails, the roomID parameter in roomInfo will be an empty string.
    // @param errorCode Error code.
    public delegate void ZIMRoomJoinedCallback(ZIMRoomFullInfo roomInfo, ZIMError errorInfo);

    // Callback of the result of entering the room.
    //
    // @param roomInfo Details of the room entered. If the join fails, the roomID parameter in roomInfo will be an empty string.
    // @param errorCode Error code.
    public delegate void ZIMRoomEnteredCallback(ZIMRoomFullInfo roomInfo, ZIMError errorInfo);

    // Callback of the result of leaving the room.
    //
    // @param errorCode Error code.
    public delegate void ZIMRoomLeftCallback(string roomID, ZIMError errorInfo);

    // Callback of the result of querying the room members list.
    //
    // @param memberList List of members in the room.
    // @param nextFlag The flag of the paging query. If this field is an empty string, the query has been completed. Otherwise, you need to set this value to the "nextFlag" field of ZIMQueryMemberConfig for the next page query.
    // @param errorCode Error code.
    public delegate void ZIMRoomMemberQueriedCallback(string roomID, List<ZIMUserInfo> memberList,
                           string nextFlag, ZIMError errorInfo);

    public delegate void ZIMRoomMembersQueriedCallback(string roomID, List<ZIMRoomMemberInfo> memberList,
    List<ZIMErrorUserInfo> errorUserList, ZIMError errorInfo);

    // Callback of the result of querying the online members count in the room.
    //
    // @param count The number of online members of the room.
    // @param errorCode Error code.
    public delegate void ZIMRoomOnlineMemberCountQueriedCallback(string roomID, uint count, ZIMError errorInfo);

    // Callback for conversation notification state settings.
    //
    // The return of the operation result of the room attribute.
    // @param errorInfo Error information.
    public delegate void ZIMRoomAttributesOperatedCallback(string roomID, List<string> errorKeyList,
                           ZIMError errorInfo);

    // Complete the property operation of the combined room.
    // @param errorInfo Error information.
    public delegate void ZIMRoomAttributesBatchOperatedCallback(string roomID, ZIMError errorInfo);

    // Return of room attribute query results.
    // @param roomAttributes Room attributes.
    // @param errorInfo Error information.
    public delegate void ZIMRoomAttributesQueriedCallback(string roomID, Dictionary<string, string> roomAttributes,
        ZIMError errorInfo);

    public delegate void ZIMRoomMembersAttributesOperatedCallback(string roomID, List<ZIMRoomMemberAttributesOperatedInfo> infos,
        List<string> errorUserList, ZIMError errorInfo);

    public delegate void ZIMRoomMembersAttributesQueriedCallback(string roomID, List<ZIMRoomMemberAttributesInfo> infos,
        ZIMError errorInfo);

    public delegate void ZIMRoomMemberAttributesListQueriedCallback(string roomID, List<ZIMRoomMemberAttributesInfo> infos,
        string nextFlag, ZIMError errorInfo);
    // Returns the result of creating a group.
    // @param groupInfo groupInfo.
    // @param userList user list.
    // @param errorUserList errorUserList.
    // @param errorInfo Error code for group creation. 0 means the group creation is successful, non-0 means the group creation fails. If there is a failure to create a group, the developer should check the official website error code document to query the solution.

    public delegate void ZIMGroupCreatedCallback(ZIMGroupFullInfo groupInfo, List<ZIMGroupMemberInfo> userList,
        List<ZIMErrorUserInfo> errorUserList, ZIMError errorInfo);

    // Returns the result of disbanding the group.
    // @param groupInfo groupInfo.
    // @param errorInfo The error code for disbanding the group. 0 means that the group is successfully disbanded, and non-0 means that the group failed to be disbanded. If there is a failure to dissolve the group, the developer should check the official website error code document to find the solution.

    public delegate void ZIMGroupDismissedCallback(string groupID, ZIMError errorInfo);

    // Returns the result of joining the group.
    // @param groupInfo groupInfo.
    // @param errorInfo Error code to join the group. 0 means joining the group successfully, non-0 means joining the group failed. If there is a failure to join the group, the developer should check the official website error code document to query the solution.

    public delegate void ZIMGroupJoinedCallback(ZIMGroupFullInfo groupInfo, ZIMError errorInfo);

    // Return of the result of leaving the group.
    // @param groupID GroupID.
    // @param errorInfo Error code for leaving the group. 0 means leaving the group successfully, non-0 means leaving the group failed. If there is a failure to leave the group, the developer should check the official website error code document to query the solution.
    public delegate void ZIMGroupLeftCallback(string groupID, ZIMError errorInfo);

    // The result of the user being invited to the group is returned.
    // @param groupID GroupID.
    // @param userList user list.
    // @param errorUserList List of users who failed to invite.
    // @param errorInfo Error code for inviting users to join the group. 0 means to invite the user to join the group, non-0 means that the invitation to join the group fails. If there is a failure to invite users to join the group, the developer should check the official website error code document to query the solution.

    public delegate void ZIMGroupUsersInvitedCallback(string groupID, List<ZIMGroupMemberInfo> userList,
        List<ZIMErrorUserInfo> errorUserList, ZIMError errorInfo);

    // Returns the result of kicking out a group member.
    // @param groupID GroupID.
    // @param kickedUserIDList List of kicked members.
    // @param errorUserList Bad user list.
    // @param errorInfo Error code for kicking out a group member. 0 means that the group member was successfully kicked out, and non-0 means that the group member failed to be kicked out. If a group member is kicked out, the developer should check the official website error code document to query the solution.
    public delegate void ZIMGroupMemberKickedCallback(string groupID, List<string> kickedUserIDList,
        List<ZIMErrorUserInfo> errorUserList, ZIMError errorInfo);

    // Returns the result of the group owner assignment.
    // @param groupID GroupID.
    // @param toUserID userID.
    // @param errorInfo The error code of the transfer group owner. 0 means the transfer of the group owner is successful, non-0 means the transfer of the group owner failed. If the transfer of the group owner fails, the developer should check the official website error code document to query the solution.

    public delegate void ZIMGroupOwnerTransferredCallback(string groupID, string toUserID, ZIMError errorInfo);

    // Returns the result of the group name update.
    // @param groupID GroupID.
    // @param groupName groupName.
    // @param errorInfo Update the error code of the group name. 0 means the update of the group name succeeds, non-0 means the update of the group name fails. If there is a failure to update the group name, the developer should check the official website error code document to query the solution.

    public delegate void ZIMGroupNameUpdatedCallback(string groupID, string groupName, ZIMError errorInfo);

    public delegate void ZIMGroupAvatarUrlUpdatedCallback(string groupID, string groupAvatarUrl, ZIMError errorInfo);

    // The return of the results of the group announcement update.
    // @param groupID GroupID.
    // @param groupNotice Group announcement.
    // @param errorInfo Update the error code of the group announcement. 0 means the update group announcement is successful, non-0 means the update group announcement fails. If the update group announcement fails, the developer should check the official website error code document to query the solution.
    public delegate void ZIMGroupNoticeUpdatedCallback(string groupID, string groupNotice, ZIMError errorInfo);

    // Returns the result of querying group information.
    // @param groupInfo groupInfo.
    // @param errorInfo Error code for querying group information. 0 means the query group information is successful, non-0 means the query group information fails. If there is a failure to query group information, the developer should check the official website for the error code document query solution.
    public delegate void ZIMGroupInfoQueriedCallback(ZIMGroupFullInfo groupInfo, ZIMError errorInfo);

    // Returns the result of a group attribute operation.
    // @param groupID GroupID.
    // @param errorKeys errorKeys.
    // @param errorInfo Set the error code of the group attribute. 0 means the group attribute setting is successful, non-0 means the group attribute setting fails. If there is a failure to set the group properties, the developer should check the official website error code documentation for solutions.
    public delegate void ZIMGroupAttributesOperatedCallback(string groupID, List<string> errorKeys,
                           ZIMError errorInfo);

    // The return of query group attribute results.
    // @param groupID GroupID.
    // @param groupAttributes groupAttributes.
    // @param errorInfo Error code for querying group attributes. 0 means the query group attribute succeeds, non-0 means the query group attribute fails. If there is a failure to query group properties, the developer should check the official website for the error code document query solution.

    public delegate void ZIMGroupAttributesQueriedCallback(string groupID, Dictionary<string, string> groupAttributes,
        ZIMError errorInfo);

    // Returns the result of the member role update.
    // @param groupID GroupID.
    // @param forUserID userID.
    // @param role ZIMGroupMemberRole.
    // @param errorInfo Error code for querying group attributes. 0 means the query group attribute succeeds, non-0 means the query group attribute fails. If there is a failure to query group properties, the developer should check the official website for the error code document query solution.

    public delegate void ZIMGroupMemberRoleUpdatedCallback(string groupID, string forUserID,
                       int role, ZIMError errorInfo);

    // Returns the result of the group member's nickname update.
    // @param groupID GroupID.
    // @param forUserID userID.
    // @param nickname userNickname.
    // @param errorInfo Set the error code of the group member's nickname. 0 means setting the group member's nickname successfully, non-0 means setting the group member's nickname failed. If there is a failure to set the group member's nickname, the developer should check the official website error code document to query the solution.

    public delegate void ZIMGroupMemberNicknameUpdatedCallback(string groupID, string forUserID,
                           string nickname, ZIMError errorInfo);

    // Returns the result of the group member query.
    // @param groupID GroupID.
    // @param userInfo Group member information.
    // @param errorInfo Query the error code of group members. 0 means querying group members successfully, non-0 means querying group members failed. If there is a failure to query group members, the developer should check the official website for the error code document query solution.

    public delegate void ZIMGroupMemberInfoQueriedCallback(string groupID, ZIMGroupMemberInfo userInfo, ZIMError errorInfo);

    // Returns the results of the query group list.
    // @param groupList groupList.
    // @param errorInfo Query the error code of the group list. 0 means the query group list succeeds, non-0 means the query group list fails. If there is a failure to query the group list, the developer should check the official website for the error code document query solution.
    public delegate void ZIMGroupListQueriedCallback(List<ZIMGroup> groupList, ZIMError errorInfo);

    // Returns the result of querying the group member list.
    // @param groupID GroupID.
    // @param userList user list.
    // @param nextFlag nextFlag.
    // @param errorInfo Error code for querying the group member list. 0 means that the query of the group member list succeeds, and non-0 means that the query of the group member list fails. If there is a failure to query the group member list, the developer should check the official website for the error code document query solution.

    public delegate void ZIMGroupMemberListQueriedCallback(string groupID, List<ZIMGroupMemberInfo> userList,
                           uint nextFlag, ZIMError errorInfo);

    public delegate void ZIMGroupMemberCountQueriedCallback(string groupID, uint count, ZIMError errorInfo);

    public delegate void ZIMGroupMessageReceiptMemberListQueriedCallback(string groupID, List<ZIMGroupMemberInfo> userList,
                           uint nextFlag, ZIMError errorInfo);

    public delegate void ZIMGroupsSearchedCallback(List<ZIMGroupSearchInfo> groupSearchInfoList,
                       uint nextFlag, ZIMError errorInfo);

    public delegate void ZIMGroupMembersSearchedCallback(string groupID, List<ZIMGroupMemberInfo> userList,
                       uint nextFlag, ZIMError errorInfo);

    // Action callback for sending a call invitation.
    // @param callID The created call invitation ID.
    // @param info Send a call invitation message.
    // @param errorInfo Error code for sending call invitation. 0 means sending the call invitation successfully, non-0 means sending the call invitation failed. If there is a failure to send a call invitation, the developer should check the official website error code documentation for solutions.

    public delegate void ZIMCallInvitationSentCallback(string callID, ZIMCallInvitationSentInfo info, ZIMError errorInfo);

    /// Supported version: 2.0.0.
    ///
    /// Detail description: Operation callback for sending a call invitation.
    ///
    /// Business scenario: After the operation of sending a call invitation is performed, the success or failure can be known through this callback.
    ///
    /// Notification timing: The result is returned after the operation of sending the call invitation is completed.
    ///
    /// Related interface: [callInvite], send a call invitation.
    public delegate void ZIMCallingInvitationSentCallback(string callID, ZIMCallingInvitationSentInfo info, ZIMError errorInfo);

    /// Quit the callback closure of the call invitation operation.
    public delegate void ZIMCallQuitSentCallback(string callID, ZIMCallQuitSentInfo info, ZIMError errorInfo);

    /// Result callback of ending the call invitation.
    public delegate void ZIMCallEndSentCallback(string callID, ZIMCallEndedSentInfo info, ZIMError errorInfo);

    /// Result Callback of querying the call list.
    public delegate void ZIMCallInvitationListQueriedCallback(List<ZIMCallInfo> callList, long nextFlag, ZIMError errorInfo);

    // Callback for the action to cancel the call invitation.
    // @param callID CallID.
    // @param errorInvitees The user who failed to cancel the call invitation.
    // @param errorInfo Error code to cancel the call invitation. 0 means canceling the call invitation successfully, non-0 means canceling the call invitation failed. If there is a failure to cancel the call invitation, the developer should check the official website error code document to query the solution.

    public delegate void ZIMCallCancelSentCallback(string callID, List<string> errorInvitees,
                           ZIMError errorInfo);

    // Action callback for accepting call invitations.
    // @param callID CallID.
    // @param errorInfo Error code to accept call invitation. 0 means accepting the call invitation successfully, non-0 means accepting the call invitation failed. If there is a failure to accept the call invitation, the developer should check the official website error code documentation for solutions.
    public delegate void ZIMCallAcceptanceSentCallback(string callID, ZIMError errorInfo);

    //Action callback for rejecting a call invitation.
    // @param callID CallID.
    // @param errorInfo Error code to reject call invitation. 0 means rejecting the call invitation successfully, non-0 means failing to reject the call invitation. If there is a failure to reject the call invitation, the developer should check the official website error code documentation for solutions.
    public delegate void ZIMCallRejectionSentCallback(string callID, ZIMError errorInfo);


}
