﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

//
//  ZIMEventHandler.h
//  ZIM
//
//  Copyright © 2021 Zego. All rights reserved.
//

namespace ZEGO
{
    // Callback
    //
    public class ZIMEventHandler
    {
        // MARK: - Main

        // The callback for error information.
        //
        // When an exception occurs in the SDK, the callback will prompt detailed information.
        //
        // @param zim ZIM instance.
        // @param errorCode Error code, please refer to the error codes document.
        // @param desc A detailed description of the error.

        public delegate void OnError(ZIM zim, ZIMError errorInfo);

        // The event callback when the connection state changes.
        //
        // @param zim ZIM instance.
        // @param state The current connection state after changed.
        // @param event The event that caused the connection state to change.
        // @param extendedData Extra information when the event occurs, a standard JSON string.

        public delegate void OnConnectionStateChanged(ZIM zim, ZIMConnectionState state,
                                              ZIMConnectionEvent connectionEvent,
                                              Dictionary<string, string> extendedData);

        // The event callback when the room connection status changes.
        //
        // Available since:  1.1.0 or above.
        //
        // Description:event callback when the room connection status changes.
        //
        // When to call::After creating a ZIM instance through [create], you can call this interface.
        //
        // Related APIs:through [onTokenWillExpire], the callback will be received when the token is about to expire.

        public delegate void OnRoomStateChanged(ZIM zim, ZIMRoomState state, ZIMRoomEvent roomEvent,
                                            Dictionary<string, string> extendedData,
                                            string roomID);

        // A reminder callback that the token is about to expire.
        //
        // When receiving this callback, you should invoke the [renewToken] function to update the token in time.
        //
        // @param zim ZIM instance.
        // @param second The remaining second before the token expires.

        public delegate void OnTokenWillExpire(ZIM zim, uint second);

        // MARK: - Message

        // The callback for receiving peer-to-peer message.
        //
        // When receiving peer-to-peer message from other user, you will receive this callback.
        //
        // @param zim ZIM instance.
        // @param messageList List of received messages.
        // @param fromUserID The user ID of the message sender.

        public delegate void
        OnReceivePeerMessage(ZIM zim,
                                 List<ZIMMessage> messageList,
                                 string fromUserID);

        // The callback for receiving room message.
        //
        // This callback will be triggered when new message is received in a room.
        //
        // @param zim ZIM instance.
        // @param messageList List of received messages.
        // @param fromRoomID ID of the room where the message was received

        public delegate void
        OnReceiveRoomMessage(ZIM zim,
                                 List<ZIMMessage> messageList,
                                 string fromRoomID);

        // The callback for receiving group message.
        //
        // This callback will be triggered when new message is received in a group.
        //
        // @param zim ZIM instance.
        // @param messageList List of received messages.
        // @param fromGroupID ID of the group where the message was received

        public delegate void
        OnReceiveGroupMessage(ZIM zim,
                                  List<ZIMMessage> messageList,
                                  string fromGroupID);

        // Received notification callback when the message receiver confirms that the message has been read.
        //
        // Available since: 2.5.0 and above.
        //
        // Description: When the message receiver confirms that the message has been read, the message sender knows through this callback.
        //
        // When to call /Trigger: Trigger a notification when the message receiver has read the message.
        //
        // Related APIs: Triggered when the peer calls via [sendMessageReceiptsRead].
        // @param zim ZIM instance.
        // @param infos Receipt information.

        public delegate void OnMessageReceiptChanged(ZIM zim,
                                                 List<ZIMMessageReceiptInfo> infos);

        // Received notification callback when some one else sends a message and then revoke a message sent by themselves.
        //
        // Available since: 2.5.0 and above.
        //
        // Description: This callback is received when some one else sends a message and then revoke.
        //
        // When to call /Trigger: This callback occurs when a ZIM instance is created with [create] and the other user revoke a message.
        //
        // Related APIs: You can revoke message to other members via [revokeMessage].
        // @param zim ZIM instance.
        // @param messageList List of received messages..

        public delegate void OnMessageRevokeReceived(
            ZIM zim, List<ZIMRevokeMessage> messageList);

        public delegate void OnMessageSentStatusChanged(
            ZIM zim,
                List<ZIMMessageSentStatusChangeInfo> messageSentStatusChangeInfoList);

        //Supported version: 2.10.0 or later.
        //
        //Description: The callback is received when reactions change
        //
        //When to Call: This callback can be registered after the ZIM instance is created by [create] and before login.
        //
        //When to Trigger: After other users add or delete reactions to messages in private or group chats.
        //
        //Related APIs: [addMessageReaction]、[deleteMessageReaction]
        // @param zim ZIM instance.
        // @param reactions The list of changed reactions.
        public delegate void OnMessageReactionsChanged(
            ZIM zim,
                List<ZIMMessageReaction> reactionList);

        // Supported version: 2.10.0 or later.
        //
        // Description: The callback is received when the server interface [SendMessageToAllUsers] sends a message.
        //
        // Call time: This callback is received after logging in to the ZIM service and the server call the [SendMessageToAllUsers] interface to send a message.
        // @param zim ZIM instance.
        // @param message Received message.
        public delegate void OnBroadcastMessageReceived(
            ZIM zim,
                ZIMMessage message);

        // MARK: - Conversation

        // Received notification callback for session update.
        //
        // Available since: 2.0.0 and above.
        //
        // Description: Trigger this callback to return notification of session updates when a session is added, deleted, or modified.
        //
        // When to call /Trigger: Notifications are triggered when a new message is updated in the session, or when the session itself is added, deleted, or modified.
        //
        // Caution: ConversationID is the same as single chat toUserID and group chat GroupID.
        //
        // Related APIs: Through [sendPeerMessage] , [sendGroupMessage], [sendRoomMessage], [deleteConversation] [deleteMessage], [deleteMessageByConversationID] trigger.
        // @param zim ZIM instance.
        // @param conversationChangeInfoList Changed information about the conversation.

        public delegate void OnConversationChanged(
            ZIM zim, List<ZIMConversationChangeInfo> conversationChangeInfoList);

        // Received notification callback for session update.
        //
        // Available since: 2.0.0 and above.
        //
        // Description: Trigger this callback to return notification of session updates when a session is added, deleted, or modified.
        //
        // When to call /Trigger: Notifications are triggered when a new message is updated in the session, or when the session itself is added, deleted, or modified.
        //
        // Caution: ConversationID is the same as single chat toUserID and group chat GroupID.
        //
        // Related APIs: Through [sendPeerMessage] , [sendGroupMessage], [sendRoomMessage], [deleteConversation] [deleteMessage], [deleteMessageByConversationID] trigger.
        // @param zim ZIM instance.
        // @param totalUnreadMessageCount Updated total unreads.
        public delegate void
        OnConversationTotalUnreadMessageCountUpdated(ZIM zim,
                                                     uint totalUnreadMessageCount);

        // Received notification callback when the message receiver has read this receipt message.
        //
        // Available since: 2.5.0 and above.
        //
        // Description: When the message receiver has read the session, the message sender knows through this callback.
        //
        // When to call /Trigger: Trigger a notification when the message receiver has read the session.
        //
        // Related APIs: triggered when the peer calls via [sendConversationMessageReceiptRead].
        // @param zim ZIM instance.
        // @param infos Receipt information.
        public delegate void
        OnConversationMessageReceiptChanged(ZIM zim,
                                                List<ZIMMessageReceiptInfo> infos);
        // MARK: - Room

        // Callback when other members join the room.
        //
        // After joining a room, this callback will be triggered when other members also join this room.
        //
        // @param zim ZIM instance.
        // @param memberList List of members who joined the room.
        // @param roomID The ID of the room where this event occurred.

        public delegate void OnRoomMemberJoined(ZIM zim, List<ZIMUserInfo> memberList,
                                            string roomID);

        // Callback when other members leave the room.
        //
        // After joining a room, this callback will be triggered when other members leave this room.
        //
        // @param zim ZIM instance.
        // @param memberList List of members who left the room.
        // @param roomID The ID of the room where this event occurred.

        public delegate void OnRoomMemberLeft(ZIM zim, List<ZIMUserInfo> memberList,
                                          string roomID);

        // Event callback when the room attributes changes.
        //
        // Available since:  1.3.0.
        //
        // Description:When the room attribute in the room changes, it will be notified through this callback.
        //
        // @param zim ZIM instance.
        // @param updateInfo The info of the room attributes changed.
        // @param roomID The ID of the room where this event occurred.

        public delegate void OnRoomAttributesUpdated(ZIM zim,
                                                 ZIMRoomAttributesUpdateInfo updateInfo,
                                                 string roomID);

        // Event callback when the room attributes changes.
        //
        // Available since:  1.3.0.
        //
        // Description:When the room attribute in the room changes, it will be notified through this callback.
        //
        // @param zim ZIM instance.
        // @param updateInfos The infos of the room attributes changed.
        // @param roomID The ID of the room where this event occurred.

        public delegate void
        OnRoomAttributesBatchUpdated(ZIM zim,
                                         List<ZIMRoomAttributesUpdateInfo> updateInfos,
                                         string roomID);

        // Room user property update callback.
        // @param ZIM instance.
        // @param infos The infos of the room member attributes changed.
        // @param operatedInfo Room operation information.
        // @param roomID Room ID.
        public delegate void OnRoomMemberAttributesUpdated(
            ZIM zim, List<ZIMRoomMemberAttributesUpdateInfo> infos,
            ZIMRoomOperatedInfo operatedInfo, string roomID);

        // MARK: - Group

        // Group state change notification callback.
        //
        // Description: allback notification of group status change.
        //
        // Use cases: Scenarios that require interaction based on the group status.
        //
        // When to call /Trigger: A notification is triggered when a group is created, joined, left, or dismissed.
        //
        // Related APIs: [createGroup] : creates a group. [joinGroup] : joins a group. [leaveGroup], leave the group. [dismissGroup]; dismiss the group.
        // @param zim ZIM instance.
        // @param state The status of the group after the change.
        // @param event Group related events.
        // @param operatedInfo Group information that has been operated.
        // @param groupInfo Information about group status changes.

        public delegate void OnGroupStateChanged(ZIM zim, ZIMGroupState state,
                                         ZIMGroupEvent groupEvent,
                                             ZIMGroupOperatedInfo operatedInfo,
                                             ZIMGroupFullInfo groupInfo);

        // Group name update notification callback.
        //
        // Description: Group name change notification callback.
        //
        // Use cases: If the group name is changed, you need to synchronize the latest group name.
        //
        // When to call /Trigger: The group name is changed. Procedure
        //
        // Related APIs: [updateGroupName] : updates the group name.
        // @param zim ZIM instance.
        // @param groupName The updated group name.
        // @param operatedInfo Operation information after the group name is updated.
        // @param groupID The groupID where the group name update occurred.

        public delegate void OnGroupNameUpdated(ZIM zim, string groupName,
                                            ZIMGroupOperatedInfo operatedInfo,
                                            string groupID);

        public delegate void OnGroupAvatarUrlUpdated(ZIM zim, string groupAvatarUrl,
                                                 ZIMGroupOperatedInfo operatedInfo,
                                                 string groupID);

        // Notification callback for group announcement updates.
        //
        // Description: Group bulletin Change notification callback.
        //
        // Use cases: If a group bulletin changes, you need to synchronize the latest bulletin content.
        //
        // When to call /Trigger: The group bulletin is changed. Procedure
        //
        // Related APIs: [updateGroupNotice], which updates the group notice.
        // @param zim ZIM instance.
        // @param groupNotice Updated group announcement.
        // @param operatedInfo The group announces the updated operation information.
        // @param groupID The groupID where the group announcement update occurred.

        public delegate void OnGroupNoticeUpdated(ZIM zim, string groupNotice,
                                              ZIMGroupOperatedInfo operatedInfo,
                                              string groupID);

        // Group attributes update notification callback.
        //
        // Description: Group attribute change notification callback.
        //
        // Use cases: When group attributes are changed, you need to synchronize the latest group attributes.
        //
        // When to call /Trigger: Triggered when group properties are set, updated, or deleted.
        //
        // Impacts on other APIs:  [setGroupAttributes] updates group attributes. [deleteGroupAttributes], delete the group attribute.
        // @param zim ZIM instance.
        // @param groupAttributes Information after group attribute update.
        // @param operatedInfo Operation information after the group attribute is updated.
        // @param groupID The groupID for sending group attribute updates.

        public delegate void OnGroupAttributesUpdated(
            ZIM zim, List<ZIMGroupAttributesUpdateInfo> groupAttributes,
                ZIMGroupOperatedInfo operatedInfo, string groupID);

        // Group member state change notification callback.
        //
        // Description: Group member status change notification callback.
        //
        // Use cases: Scenarios that require interaction based on group member states.
        //
        // When to call /Trigger: Notification is triggered when a group is created, joined, left, or dismissed, or a user is invited to join or kicked out of the group.
        //
        // Related APIs: [createGroup] : creates a group. [joinGroup] : joins a group. [leaveGroup], leave the group. [dismissGroup]; dismiss the group. [intiveUsersIntoGroup], which invites users to join the group. [kickoutGroupMember] kicks the user out of the group.
        // @param zim ZIM instance.
        // @param state Updated membership status.
        // @param event Updated member events.
        // @param userList Updated member information.
        // @param operatedInfo Updated operational information.
        // @param groupID The groupID where the member state change occurred.

        public delegate void OnGroupMemberStateChanged(ZIM zim, ZIMGroupMemberState state,
                                               ZIMGroupMemberEvent groupMemberEvent,
                                                   List<ZIMGroupMemberInfo> userList,
                                                   ZIMGroupOperatedInfo operatedInfo,
                                                   string groupID);

        // Group member information change notification.
        //
        // Description: Return the operation result of changing group member information.
        //
        // Use cases: After the basic information of group members is changed, you need to display or interact with group members on the page.
        //
        // When to call /Trigger: The result is displayed after the group member information is changed.
        //
        // Related APIs: [setGroupMemberNickname] : updates the nickname of a group member. [setGroupMemberRole] : updates the group member role. [transferGroupOwner], group master transfer.
        // @param zim ZIM instance.
        // @param groupMemberInfos Updated member information.
        // @param operatedInfo Updated operational information.
        // @param groupID The groupID where the group member information has been changed.

        public delegate void OnGroupMemberInfoUpdated(
            ZIM zim, List<ZIMGroupMemberInfo> groupMemberInfos,
                ZIMGroupOperatedInfo operatedInfo, string groupID);

        // MARK: - CallInvite

        // The notification callback for the call invitation received by the invitee.
        //
        // Supported versions: 2.0.0 and above.
        //
        // Detail description: After the inviter initiates a call invitation, the invitee will receive this callback when the invitee is online.
        //
        // Business scenario: The invitee will call this callback after the inviter sends a call invitation.
        //
        // When to call: After creating a ZIM instance through [create].
        //
        // Note: If the user is not in the invitation list or not online, this callback will not be called.
        // @param zim ZIM instance.
        // @param info Information about received call invitations.
        // @param callID Received CallID.

        public delegate void OnCallInvitationReceived(ZIM zim,
                                                  ZIMCallInvitationReceivedInfo info,
                                                  string callID);

        // The notification callback received by the invitee to cancel the invitation.
        //
        // Supported versions: 2.0.0 and above.
        //
        // Detail description: After the inviter cancels the call invitation, this callback will be received when the invitee is online.
        //
        // Business scenario: The invitee will call this callback after the inviter cancels the call invitation.
        //
        // When to call: After creating a ZIM instance through [create].
        //
        // Note: If the user is not in the cancel invitation list or is offline, this callback will not be called.
        //
        // Related interface: [callCancel].
        // @param zim ZIM instance.
        // @param info Information about canceled call invitations.
        // @param callID Cancelled callID.

        public delegate void OnCallInvitationCancelled(ZIM zim,
                                                   ZIMCallInvitationCancelledInfo info,
                                                   string callID);

        // The notification callback received by the inviter that invitee has accepted the invitation.
        //
        // Supported versions: 2.0.0 and above.
        //
        // Detail description: After the invitee accepts the call invitation, this callback will be received when the inviter is online.
        //
        // Business scenario: The inviter will receive this callback after the inviter accepts the call invitation.
        //
        // When to call: After creating a ZIM instance through [create].
        //
        // Note: This callback will not be called if the user is not online.
        //
        // Related interface: [callAccept].
        //
        // @param zim ZIM instance.
        // @param info Information about the call invitation.
        // @param callID The accepted callID.
        [Obsolete("Deprecated since 2.9.0, pleace use  [onCallUserStateChanged] instead.")]
        public delegate void OnCallInvitationAccepted(ZIM zim,
                                                  ZIMCallInvitationAcceptedInfo info,
                                                  string callID);

        // The inviter receives the notification callback that the invitee reject the invitation.
        //
        // Available since: 2.0.0 and above.
        //
        // Description: This callback will be received when the inviter is online after the inviter rejects the call invitation.
        //
        // Use cases: The inviter will receive this callback after the inviter declines the call invitation.
        //
        // Default value: After creating a ZIM instance through [create] and logging in.
        //
        // When to call /Trigger: After creating a ZIM instance through [create] and logging in.
        //
        // Restrictions: If the user is not the inviter of the call invitation or is not online, the callback will not be received.
        //
        // Related APIs:[callReject].
        // @param zim ZIM instance.
        // @param info Information about rejected call invitations.
        // @param callID The callID of the rejected call invitation.

        [Obsolete("Deprecated since 2.9.0, pleace use [onCallUserStateChanged] instead.")]
        public delegate void OnCallInvitationRejected(ZIM zim,
                                                  ZIMCallInvitationRejectedInfo info,
                                                   string callID);

        // Supported versions: 2.0.0 and above.
        //
        // Detail description: When the call invitation times out, the invitee does not respond and will receive a callback.
        //
        // Business scenario: If the invitee does not respond before the timeout period, this callback will be received.
        //
        // When to call: After creating a ZIM instance through [create].
        //
        // Note: If the user is not on the invitation list or is not online, the callback will not be received.
        //
        // Related interfaces: [callInvite], [callAccept], [callReject].
        // @param zim ZIM instance.
        // @param callID The callID of the timeout.

        public delegate void OnCallInvitationTimeout(ZIM zim, ZIMCallInvitationTimeoutInfo info, string callID);

        // Notification callback of call invitation has timeouted.
        //
        // Supported versions: 2.0.0 and above.
        //
        // Detail description: When the call invitation times out, the invitee does not respond, and the inviter will receive a callback.
        //
        // Business scenario: The invitee does not respond before the timeout period, and the inviter will receive this callback.
        //
        // When to call: After creating a ZIM instance through [create].
        //
        // Note: If the user is not the inviter who initiated this call invitation or is not online, the callback will not be received.
        //
        // Related interfaces: [callInvite], [callAccept], [callReject].
        // @param zim ZIM instance.
        // @param invitees Timeout invitee ID.
        // @param callID The number of invitees that have timed out.

        [Obsolete("Deprecated since 2.9.0, pleace use [onCallUserStateChanged] instead.")]
        public delegate void OnCallInviteesAnsweredTimeout(ZIM zim,
                                                        List<string> invitees,
                                                        string callID);

        /// Supported versions: 2.9.0 and above.
        ///
        /// Detail description: In an advanced call,  a participant ends the call, and all participants will receive this callback.
        ///
        /// Note: If the user is not the inviter who initiated this call invitation or is not online, the callback will not be received.
        ///
        /// Related APIs: [callEnd]
        ///
        /// @param zim ZIM instance.
        /// @param info Information carried by the event callback.
        /// @param callID Timeout invitee ID.
        public delegate void OnCallInvitationEnded(ZIM zim, ZIMCallInvitationEndedInfo info,
                                       string callID);


        public delegate void OnCallUserStateChanged(ZIM zim, ZIMCallUserStateChangeInfo info,
                                        string  callID);

    };

} // namespace zim

